@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: if_athvar.h 2952 2007-11-25 04:08:38Z mentor $
+ * $Id: if_athvar.h 3202 2008-01-18 01:28:43Z mentor $
  */
 
 /*
@@ -193,7 +193,17 @@ static inline struct net_device *_alloc_netdev(int sizeof_priv, const char *mask
 
 #define	ATH_TIMEOUT	1000
 
-#define ATH_DFS_WAIT_POLL_PERIOD	2	/* 2 seconds */
+#define	ATH_DFS_WAIT_MIN_PERIOD		60	/* DFS wait is 60 seconds, per
+						 * FCC/ETSI regulations. */
+
+#define	ATH_DFS_WAIT_SHORT_POLL_PERIOD	2	/* 2 seconds, for consecutive
+						 * waits if not done yet. */
+
+#define	ATH_DFS_AVOID_MIN_PERIOD	1800	/* 30 minutes, per FCC/ETSI
+						 * regulations */
+
+#define	ATH_DFS_TEST_RETURN_PERIOD	15	/* 15 seconds -- for mute test
+						 * only */
 
 #define	ATH_LONG_CALINTERVAL		30	/* 30 seconds between calibrations */
 #define	ATH_SHORT_CALINTERVAL		1	/* 1 second between calibrations */
@@ -209,7 +219,9 @@ static inline struct net_device *_alloc_netdev(int sizeof_priv, const char *mask
 #define	ATH_RXBUF	40		/* number of RX buffers */
 #define	ATH_TXBUF	200		/* number of TX buffers */
 
-#define	ATH_BCBUF	4		/* number of beacon buffers */
+#define ATH_MAXVAPS_MIN 	2	/* minimum number of beacon buffers */
+#define ATH_MAXVAPS_MAX 	64	/* maximum number of beacon buffers */
+#define ATH_MAXVAPS_DEFAULT 	4	/* default number of beacon buffers */
 
 /* free buffer threshold to restart net dev */
 #define	ATH_TXBUF_FREE_THRESHOLD  (ATH_TXBUF / 20)
@@ -606,38 +618,44 @@ struct ath_softc {
 	struct ath_ratectrl *sc_rc;		/* tx rate control support */
 	struct ath_tx99 *sc_tx99; 		/* tx99 support */
 	void (*sc_setdefantenna)(struct ath_softc *, u_int);
-	unsigned int 	sc_invalid:1,		/* being detached */
-			sc_mrretry:1,		/* multi-rate retry support */
-			sc_softled:1,		/* enable LED gpio status */
-			sc_splitmic:1,		/* split TKIP MIC keys */
-			sc_needmib:1,		/* enable MIB stats intr */
-			sc_hasdiversity:1,	/* rx diversity available */
-			sc_diversity:1, 	/* enable rx diversity */
-			sc_olddiversity:1, 	/* diversity setting before XR enable */
-			sc_hasveol:1,		/* tx VEOL support */
-			sc_hastpc:1,		/* per-packet TPC support */
-			sc_dturbo:1,		/* dynamic turbo capable */
-			sc_dturbo_switch:1,	/* turbo switch mode*/
-			sc_dturbo_hold:1,	/* dynamic turbo hold state */
-			sc_rate_recn_state:1,	/* dynamic turbo state recmded by ratectrl */
-			sc_ignore_ar:1,		/* ignore AR during transition */
-			sc_ledstate:1,		/* LED on/off state */
-			sc_blinking:1,		/* LED blink operation active */
-			sc_beacons:1,		/* beacons running */
-			sc_hasbmask:1,		/* bssid mask support */
-			sc_mcastkey:1,		/* mcast key cache search */
-			sc_hastsfadd:1,		/* tsf adjust support */
-			sc_scanning:1,		/* scanning active */
-			sc_nostabeacons:1,	/* no beacons for station */
-			sc_xrgrppoll:1,		/* xr group polls are active */
-			sc_syncbeacon:1,	/* sync/resync beacon timers */
-			sc_hasclrkey:1,		/* CLR key supported */
-			sc_devstopped:1,	/* stopped due to of no tx bufs */
-			sc_stagbeacons:1,	/* use staggered beacons */
-			sc_dfswait:1,		/* waiting on channel for radar detect */
-			sc_ackrate:1,		/* send acks at high bitrate */
-			sc_hasintmit:1,		/* Interference mitigation */
-			sc_txcont:1;        	/* Is continuous transmit enabled? */
+
+	unsigned int 	sc_invalid:1;		/* being detached */
+	unsigned int	sc_mrretry:1;		/* multi-rate retry support */
+	unsigned int	sc_softled:1;		/* enable LED gpio status */
+	unsigned int	sc_splitmic:1;		/* split TKIP MIC keys */
+	unsigned int	sc_needmib:1;		/* enable MIB stats intr */
+	unsigned int	sc_hasdiversity:1;	/* rx diversity available */
+	unsigned int	sc_diversity:1;		/* enable rx diversity */
+	unsigned int	sc_olddiversity:1;	/* diversity setting before XR enable */
+	unsigned int	sc_hasveol:1;		/* tx VEOL support */
+	unsigned int	sc_hastpc:1;		/* per-packet TPC support */
+	unsigned int	sc_dturbo:1;		/* dynamic turbo capable */
+	unsigned int	sc_dturbo_switch:1;	/* turbo switch mode*/
+	unsigned int	sc_dturbo_hold:1;	/* dynamic turbo hold state */
+	unsigned int	sc_rate_recn_state:1;	/* dynamic turbo state recmded by ratectrl */
+	unsigned int	sc_ignore_ar:1;		/* ignore AR during transition */
+	unsigned int	sc_ledstate:1;		/* LED on/off state */
+	unsigned int	sc_blinking:1;		/* LED blink operation active */
+	unsigned int	sc_beacons:1;		/* beacons running */
+	unsigned int	sc_hasbmask:1;		/* bssid mask support */
+	unsigned int	sc_mcastkey:1;		/* mcast key cache search */
+	unsigned int	sc_hastsfadd:1;		/* tsf adjust support */
+	unsigned int	sc_scanning:1;		/* scanning active */
+	unsigned int	sc_nostabeacons:1;	/* no beacons for station */
+	unsigned int	sc_xrgrppoll:1;		/* xr group polls are active */
+	unsigned int	sc_syncbeacon:1;	/* sync/resync beacon timers */
+	unsigned int	sc_hasclrkey:1;		/* CLR key supported */
+	unsigned int	sc_devstopped:1;	/* stopped due to of no tx bufs */
+	unsigned int	sc_stagbeacons:1;	/* use staggered beacons */
+	unsigned int	sc_dfswait:1;		/* waiting on channel for radar detect */
+	unsigned int	sc_ackrate:1;		/* send acks at high bitrate */
+	unsigned int	sc_dfs_cac:1;		/* waiting on channel for radar detect */
+	unsigned int	sc_hasintmit:1;		/* Interference mitigation */
+	unsigned int	sc_txcont:1;		/* Is continuous transmit enabled? */
+	unsigned int	sc_dfs_testmode:1; 	/* IF this is on, AP vaps will stay in
+						 * 'channel availability check' indefinately,
+						 * reporting radar and interference detections.
+						 */
 	unsigned int sc_txcont_power; /* Continuous transmit power in 0.5dBm units */
 	unsigned int sc_txcont_rate;  /* Continuous transmit rate in Mbps */
 
@@ -729,7 +747,7 @@ struct ath_softc {
 		COMMIT				/* beacon sent, commit change */
 	} sc_updateslot;			/* slot time update fsm */
 	int sc_slotupdate;			/* slot to next advance fsm */
-	struct ieee80211vap *sc_bslot[ATH_BCBUF];/* beacon xmit slots */
+	struct ieee80211vap **sc_bslot;		/* beacon xmit slots */
 	int sc_bnext;				/* next slot for beacon xmit */
 
 	struct timer_list sc_cal_ch;		/* calibration timer */
@@ -759,6 +777,8 @@ struct ath_softc {
 #endif
 	u_int sc_slottimeconf;			/* manual override for slottime */
 	u_int64_t sc_tsf;			/* TSF at last rx interrupt */
+
+	u_int32_t sc_nexttbtt;
 };
 
 typedef void (*ath_callback) (struct ath_softc *);
@@ -844,5 +864,8 @@ int ath_ioctl_ethtool(struct ath_softc *, int, void __user *);
 void bus_read_cachesize(struct ath_softc *, u_int8_t *);
 void ath_sysctl_register(void);
 void ath_sysctl_unregister(void);
+int ar_device(int devid);
+#define DEV_NAME(_d) ((NULL == _d || NULL == _d->name || 0 == strncmp(_d->name, "wifi%d", 6)) ? "MadWifi" : _d->name)
+
 
 #endif /* _DEV_ATH_ATHVAR_H */
