@@ -446,7 +446,6 @@ ath_rate_tx_complete(struct ath_softc *sc,
 		const struct ath_desc *ds = &bf->bf_desc[0];
 		int final_rate = 0;
 		int tries = 0;
-		int ndx = -1;
 		int mrr;
 		int final_ndx;
 		int rate0, tries0, ndx0;
@@ -485,8 +484,8 @@ ath_rate_tx_complete(struct ath_softc *sc,
 		mrr = sc->sc_mrretry && !(ic->ic_flags & IEEE80211_F_USEPROT) && ENABLE_MRR;
 
 		if (!mrr) {
-			if ((0 <= ndx) && (ndx < sn->num_rates)) {
-				sn->rs_rateattempts[ndx]++; /* only one rate was used */
+			if ((0 <= final_ndx) && (final_ndx < sn->num_rates)) {
+				sn->rs_rateattempts[final_ndx] += tries; /* only one rate was used */
 			}
 			return;
 		}
