@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211.c 2988 2007-11-28 05:36:29Z proski $
+ * $Id: ieee80211.c 3235 2008-01-23 18:53:34Z mtaylor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -661,7 +661,8 @@ ieee80211_find_channel(struct ieee80211com *ic, int freq, int flags)
 	for (i = 0; i < ic->ic_nchans; i++) {
 		c = &ic->ic_channels[i];
 		if (c->ic_freq == freq &&
-		    (c->ic_flags & IEEE80211_CHAN_ALLTURBO) == flags)
+		    (flags == 0 || 
+		     (c->ic_flags & IEEE80211_CHAN_ALLTURBO) == flags))
 			return c;
 	}
 	return NULL;
@@ -840,7 +841,7 @@ ieee80211_mark_dfs(struct ieee80211com *ic, struct ieee80211_channel *ichan)
 		} else {
 			/* Change to a radar free 11a channel for dfstesttime seconds */
 			ic->ic_chanchange_chan = IEEE80211_RADAR_TEST_MUTE_CHAN;
-			ic->ic_chanchange_tbtt = IEEE80211_RADAR_11HCOUNT;
+			ic->ic_chanchange_tbtt = IEEE80211_RADAR_CHANCHANGE_TBTT_COUNT;
 			ic->ic_flags |= IEEE80211_F_CHANSWITCH;
 			/* A timer is setup in the radar task if markdfs is not set and
 			 * we are in hostap mode.
@@ -861,7 +862,7 @@ ieee80211_dfs_test_return(struct ieee80211com *ic, u_int8_t ieeeChan)
 	if_printf(dev, "Returning to channel %d\n", ieeeChan);
 	printk("Returning to chan %d\n", ieeeChan);
 	ic->ic_chanchange_chan = ieeeChan;
-	ic->ic_chanchange_tbtt = IEEE80211_RADAR_11HCOUNT;
+	ic->ic_chanchange_tbtt = IEEE80211_RADAR_CHANCHANGE_TBTT_COUNT;
 	ic->ic_flags |= IEEE80211_F_CHANSWITCH;
 }
 EXPORT_SYMBOL(ieee80211_dfs_test_return);
