@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_node.c 2964 2007-11-25 09:09:24Z mtaylor $
+ * $Id: ieee80211_node.c 3140 2008-01-16 00:08:14Z mtaylor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -306,12 +306,15 @@ ieee80211_create_ibss(struct ieee80211vap* vap, struct ieee80211_channel *chan)
 		"%s: creating ibss on channel %u\n", __func__,
 		ieee80211_chan2ieee(ic, chan));
 
-	/* Check to see if we already have a node for this mac 
+	/* Check to see if we already have a node for this mac
 	 * NB: we gain a node reference here
 	 */
 	ni = ieee80211_find_node(&ic->ic_sta, vap->iv_myaddr);
 	if (ni == NULL) {
 		ni = ieee80211_alloc_node_table(vap, vap->iv_myaddr);
+		IEEE80211_DPRINTF(vap, IEEE80211_MSG_ASSOC,
+				  "%s: ni:%p allocated for " MAC_FMT "\n",
+				  __func__, ni, MAC_ADDR(vap->iv_myaddr));
 		if (ni == NULL) {
 			/* XXX recovery? */
 			return;
@@ -401,6 +404,9 @@ ieee80211_reset_bss(struct ieee80211vap *vap)
 	ieee80211_reset_erp(ic, ic->ic_curmode);
 
 	ni = ieee80211_alloc_node_table(vap, vap->iv_myaddr);
+	IEEE80211_DPRINTF(vap, IEEE80211_MSG_ASSOC,
+			  "%s: ni:%p allocated for " MAC_FMT "\n",
+			  __func__, ni, MAC_ADDR(vap->iv_myaddr));
 	KASSERT(ni != NULL, ("unable to setup inital BSS node"));
 
 	/* New reference for caller */
@@ -551,12 +557,17 @@ ieee80211_ibss_merge(struct ieee80211_node *ni)
 #ifdef IEEE80211_DEBUG
 	struct ieee80211com *ic = ni->ni_ic;
 #endif
+	IEEE80211_DPRINTF(vap, IEEE80211_MSG_ASSOC,
+			  "%s: ni:%p[" MAC_FMT "] iv_bss:%p[" MAC_FMT "]\n",
+			  __func__, ni, MAC_ADDR(ni->ni_macaddr),
+			  vap->iv_bss, MAC_ADDR(vap->iv_bss->ni_macaddr));
 
 	if (ni == vap->iv_bss ||
 	    IEEE80211_ADDR_EQ(ni->ni_bssid, vap->iv_bss->ni_bssid)) {
 		/* unchanged, nothing to do */
 		return 0;
 	}
+
 	if (!check_bss(vap, ni)) {
 		/* capabilities mismatch */
 		IEEE80211_DPRINTF(vap, IEEE80211_MSG_ASSOC,
@@ -614,8 +625,8 @@ ieee80211_sta_join1(struct ieee80211_node *selbs)
 	 * Check if old+new node have the same ssid in which
 	 * case we can reassociate when operating in sta mode.
 	 */
-	canreassoc = (obss != NULL &&
-		vap->iv_state == IEEE80211_S_RUN && ssid_equal(obss, selbs));
+	canreassoc = ((obss != NULL) &&
+		(vap->iv_state == IEEE80211_S_RUN) && ssid_equal(obss, selbs));
 	vap->iv_bss = selbs;
 	if (obss != NULL)
 		ieee80211_unref_node(&obss);
@@ -676,6 +687,9 @@ ieee80211_sta_join(struct ieee80211vap *vap,
 	ni = ieee80211_find_node(&ic->ic_sta, se->se_macaddr);
 	if (ni == NULL) {
 		ni = ieee80211_alloc_node_table(vap, se->se_macaddr);
+		IEEE80211_DPRINTF(vap, IEEE80211_MSG_ASSOC,
+				  "%s: ni:%p allocated for " MAC_FMT "\n",
+				  __func__, ni, MAC_ADDR(se->se_macaddr));
 		if (ni == NULL) {
 			IEEE80211_DPRINTF(vap, IEEE80211_MSG_NODE,
 			"%s: Unable to allocate node for BSS: %s\n", __func__,
@@ -734,8 +748,8 @@ ieee80211_sta_leave(struct ieee80211_node *ni)
 	struct ieee80211vap *vap = ni->ni_vap;
 
 	/* WDS/Repeater: Stop software beacon timer for STA */
-	if (vap->iv_opmode == IEEE80211_M_STA &&
-	    vap->iv_flags_ext & IEEE80211_FEXT_SWBMISS) {
+	if ((vap->iv_opmode == IEEE80211_M_STA) &&
+			(vap->iv_flags_ext & IEEE80211_FEXT_SWBMISS)) {
 		del_timer(&vap->iv_swbmiss);
 	}
 
@@ -1036,6 +1050,9 @@ ieee80211_alloc_node_table(struct ieee80211vap *vap,
 #endif
 		IEEE80211_NODE_TABLE_UNLOCK_IRQ(nt);
 	}
+	else {
+		printk("Failed to allocate node for %s.\n", ether_sprintf(macaddr));
+	}
 
 	return ni;
 }
@@ -1263,6 +1280,9 @@ ieee80211_dup_bss(struct ieee80211vap *vap, const u_int8_t *macaddr,
 #else
 		ni = ieee80211_alloc_node_table(vap, macaddr);
 #endif
+		IEEE80211_DPRINTF(vap, IEEE80211_MSG_ASSOC,
+				  "%s: ni:%p allocated for " MAC_FMT "\n",
+				  __func__, ni, MAC_ADDR(macaddr));
 	}
 
 	if (ni != NULL) {
@@ -1745,8 +1765,14 @@ restart:
 		/*
 		 * Special case ourself; we may be idle for extended periods
 		 * of time and regardless reclaiming our state is wrong.
-		 */
-		if (ni == ni->ni_vap->iv_bss) {
+		 * Special case a WDS link: it may be dead or idle, but it is 
+		 * never ok to reclaim it, as this will block transmissions
+		 * and nobody will recreate the node when the WDS peer is
+		 * available again. */
+		if ((ni == ni->ni_vap->iv_bss) ||
+		    (ni->ni_vap->iv_opmode == IEEE80211_M_WDS && 
+		     !memcmp(ni->ni_macaddr, ni->ni_vap->wds_mac, ETH_ALEN)))
+		{
 			/* NB: don't permit it to go negative */
 			if (ni->ni_inact > 0)
 				ni->ni_inact--;
