@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_node.h 2915 2007-11-22 02:08:35Z mentor $
+ * $Id: ieee80211_node.h 3223 2008-01-20 03:34:45Z mentor $
  */
 #ifndef _NET80211_IEEE80211_NODE_H_
 #define _NET80211_IEEE80211_NODE_H_
@@ -187,12 +187,15 @@ MALLOC_DECLARE(M_80211_NODE);
 #define	IEEE80211_NODE_STAT_ADD(ni,stat,v)	(ni->ni_stats.ns_##stat += v)
 #define	IEEE80211_NODE_STAT_SET(ni,stat,v)	(ni->ni_stats.ns_##stat = v)
 
-#define WME_UAPSD_AC_CAN_TRIGGER(_ac, _ni) ( \
-		((_ni)->ni_flags & IEEE80211_NODE_UAPSD_TRIG) && WME_UAPSD_AC_ENABLED((_ac), (_ni)->ni_uapsd) )
+#define WME_UAPSD_AC_CAN_TRIGGER(_ac, _ni) (				\
+		((_ni)->ni_flags & IEEE80211_NODE_UAPSD_TRIG) &&	\
+		WME_UAPSD_AC_ENABLED((_ac), (_ni)->ni_uapsd))
 #define WME_UAPSD_NODE_MAXQDEPTH	8
-#define IEEE80211_NODE_UAPSD_USETIM(_ni) (((_ni)->ni_uapsd & 0xF) == 0xF )
+#define IEEE80211_NODE_UAPSD_USETIM(_ni) (((_ni)->ni_uapsd & 0xF) == 0xF)
 #define WME_UAPSD_NODE_INVALIDSEQ	0xffff
-#define WME_UAPSD_NODE_TRIGSEQINIT(_ni)	(memset(&(_ni)->ni_uapsd_trigseq[0], 0xff, sizeof((_ni)->ni_uapsd_trigseq)))
+#define WME_UAPSD_NODE_TRIGSEQINIT(_ni)					\
+		(memset(&(_ni)->ni_uapsd_trigseq[0],			\
+		 0xff, sizeof((_ni)->ni_uapsd_trigseq)))
 
 void ieee80211_node_attach(struct ieee80211com *);
 void ieee80211_node_detach(struct ieee80211com *);
@@ -343,7 +346,7 @@ ieee80211_pass_node(struct ieee80211_node **pni) {
 	return (tmp);
 }
 
-/* Decrement ieee80211_node* refcunt, and relinquish the pointer. */
+/* Decrement ieee80211_node* refcount, and relinquish the pointer. */
 #ifdef IEEE80211_DEBUG_REFCNT
 #define ieee80211_unref_node(_pni) \
 	ieee80211_unref_node_debug(_pni, __func__, __LINE__)
