@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_output.c 3002 2007-12-03 08:52:34Z br1 $
+ * $Id: ieee80211_output.c 3167 2008-01-16 20:11:38Z mentor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -196,6 +196,7 @@ ieee80211_classify(struct ieee80211_node *ni, struct sk_buff *skb)
 
 /*
  * Context: process context (BHs disabled)
+ * It must return either NETDEV_TX_OK or NETDEV_TX_BUSY
  */
 int
 ieee80211_hardstart(struct sk_buff *skb, struct net_device *dev)
@@ -233,7 +234,7 @@ ieee80211_hardstart(struct sk_buff *skb, struct net_device *dev)
 	if (vap->iv_opmode == IEEE80211_M_MONITOR) {
 		ieee80211_monitor_encap(vap, skb);
 		ieee80211_parent_queue_xmit(skb);
-		return 0;
+		return NETDEV_TX_OK;
 	}
 	
 	/* Cancel any running BG scan */
@@ -248,11 +249,11 @@ ieee80211_hardstart(struct sk_buff *skb, struct net_device *dev)
 		ni = ieee80211_find_txnode(vap, vap->wds_mac);
 	else
 		ni = ieee80211_find_txnode(vap, eh->ether_dhost);
-
 	if (ni == NULL) {
 		/* NB: ieee80211_find_txnode does stat+msg */
 		goto bad;
 	}
+
 	/* calculate priority so drivers can find the TX queue */
 	if (ieee80211_classify(ni, skb)) {
 		IEEE80211_NOTE(vap, IEEE80211_MSG_OUTPUT, ni,
@@ -268,23 +269,21 @@ ieee80211_hardstart(struct sk_buff *skb, struct net_device *dev)
 		/* XXXAPSD: assuming triggerable means deliverable */
 		M_FLAG_SET(skb, M_UAPSD);
 	} else if ((ni->ni_flags & IEEE80211_NODE_PWR_MGT)) {
-		/*
-		 * Station in power save mode; stick the frame
+		/* Station in power save mode; stick the frame
 		 * on the STA's power save queue and continue.
-		 * We'll get the frame back when the time is right.
-		 */
-		ieee80211_pwrsave(ni, skb);
+		 * We'll get the frame back when the time is right. */
 		ieee80211_unref_node(&ni);
-		return 0;
+		return ieee80211_pwrsave(skb);
 	}
 
+	dev->trans_start = jiffies;
+
 #ifdef ATH_SUPERG_XR
 	/* Broadcast/multicast packets need to be sent on XR vap in addition to
 	 * normal vap. */
-
 	if (vap->iv_xrvap && (ni == vap->iv_bss) &&
 	    vap->iv_xrvap->iv_sta_assoc) {
-		struct sk_buff *skb1 = skb_clone(skb, GFP_ATOMIC);
+		struct sk_buff *skb1 = skb_copy(skb, GFP_ATOMIC);
 		if (skb1) {
 			memset(SKB_CB(skb1), 0, sizeof(struct ieee80211_cb));
 #ifdef IEEE80211_DEBUG_REFCNT
@@ -292,25 +291,29 @@ ieee80211_hardstart(struct sk_buff *skb, struct net_device *dev)
 #endif /* #ifdef IEEE80211_DEBUG_REFCNT */
 			SKB_CB(skb1)->ni = ieee80211_find_txnode(vap->iv_xrvap, 
 						       eh->ether_dhost);
+			/* Ignore this return code. */
 			ieee80211_parent_queue_xmit(skb1);
 		}
 	}
 #endif
-	ieee80211_parent_queue_xmit(skb);
 	ieee80211_unref_node(&ni);
-	return 0;
+	ieee80211_parent_queue_xmit(skb);
+	return NETDEV_TX_OK;
 
 bad:
 	if (skb != NULL)
 		ieee80211_dev_kfree_skb(&skb);
 	if (ni != NULL)
 		ieee80211_unref_node(&ni);
-	return 0;
+	return NETDEV_TX_OK;
 }
 
+/*
+ * skb is consumed in all cases
+ */
+
 void ieee80211_parent_queue_xmit(struct sk_buff *skb) {
 	struct ieee80211vap *vap = skb->dev->priv;
-	struct ieee80211_node *ni;
 
 	vap->iv_devstats.tx_packets++;
 	vap->iv_devstats.tx_bytes += skb->len;
@@ -319,21 +322,9 @@ void ieee80211_parent_queue_xmit(struct sk_buff *skb) {
 	/* Dispatch the packet to the parent device */
 	skb->dev = vap->iv_ic->ic_dev;
 
-	ni = SKB_CB(skb)->ni;
-	if ( dev_queue_xmit(skb) == NET_XMIT_DROP ) {
-		/* If queue dropped the packet because device was
-		 * too busy */
+	if (dev_queue_xmit(skb) == NET_XMIT_DROP)
 		vap->iv_devstats.tx_dropped++;
-		if (ni != NULL) {
-			/* node reference was leaked */
-			ieee80211_unref_node(&ni);
-		}
-	}
-	else {
-		/* node reference was not leaked, forget about it */
-		ni = NULL;
-	}
-	skb = NULL; /* skb is no longer ours */
+
 }
 
 /*
@@ -478,7 +469,7 @@ ieee80211_send_nulldata(struct ieee80211_node *ni)
 		skb_push(skb, sizeof(struct ieee80211_frame));
 	ieee80211_send_setup(vap, ni, wh,
 		IEEE80211_FC0_TYPE_DATA | IEEE80211_FC0_SUBTYPE_NODATA,
-		vap->iv_myaddr, ni->ni_macaddr, ni->ni_bssid);
+		vap->iv_myaddr, ni->ni_macaddr, vap->iv_bss->ni_bssid);
 	/* NB: power management bit is never sent by an AP */
 	if ((IEEE80211_VAP_IS_SLEEPING(ni->ni_vap)) &&
 	    vap->iv_opmode != IEEE80211_M_HOSTAP &&
@@ -533,7 +524,7 @@ ieee80211_send_qosnulldata(struct ieee80211_node *ni, int ac)
 		IEEE80211_FC0_TYPE_DATA,
 		vap->iv_myaddr, /* SA */
 		ni->ni_macaddr, /* DA */
-		ni->ni_bssid);
+		vap->iv_bss->ni_bssid);
 
 	qwh->i_fc[0] = IEEE80211_FC0_VERSION_0 | IEEE80211_FC0_TYPE_DATA |
 		IEEE80211_FC0_SUBTYPE_QOS_NULL;
@@ -706,9 +697,8 @@ ieee80211_skbhdr_adjust(struct ieee80211vap *vap, int hdrsize,
 		int n = 0;
 		if (need_headroom > skb_headroom(skb))
 			n = need_headroom - skb_headroom(skb);
-		if (pskb_expand_head(skb, n,
-			need_tailroom - skb_tailroom(skb), GFP_ATOMIC)) {
-			ieee80211_dev_kfree_skb(&skb);
+		if (pskb_expand_head(skb, n, need_tailroom - 
+					skb_tailroom(skb), GFP_ATOMIC)) {
 			IEEE80211_DPRINTF(vap, IEEE80211_MSG_OUTPUT,
 				"%s: cannot expand storage (tail)\n", __func__);
 			vap->iv_stats.is_tx_nobuf++;
@@ -1754,7 +1744,7 @@ ieee80211_send_probereq(struct ieee80211_node *ni,
 		ether_sprintf(wh->i_addr1),
 		ieee80211_chan2ieee(ic, ic->ic_curchan));
 
-	(void) ic->ic_mgtstart(ic, skb);
+	(void)ic->ic_mgtstart(ic, skb);
 	return 0;
 }
 
