@@ -28,7 +28,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_scan.c 2892 2007-11-21 11:20:36Z mtaylor $
+ * $Id: ieee80211_scan.c 3162 2008-01-16 20:01:39Z mtaylor $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -645,7 +645,7 @@ scan_next(unsigned long arg)
 
 again:
 	scandone = (ss->ss_next >= ss->ss_last) ||
-		(SCAN_PRIVATE(ss)->ss_iflags & ISCAN_CANCEL) != 0;
+		((SCAN_PRIVATE(ss)->ss_iflags & ISCAN_CANCEL) != 0);
 	scanend = SCAN_PRIVATE(ss)->ss_scanend;
 	if (!scandone &&
 	    (ss->ss_flags & IEEE80211_SCAN_GOTPICK) == 0 &&
@@ -719,8 +719,8 @@ again:
 		if (scandone)
 			ic->ic_lastscan = jiffies;
 		/* return to the bss channel */
-		if (ic->ic_bsschan != IEEE80211_CHAN_ANYC &&
-		    ic->ic_curchan != ic->ic_bsschan)
+		if ((ic->ic_bsschan != IEEE80211_CHAN_ANYC) &&
+		    (ic->ic_curchan != ic->ic_bsschan))
 			change_channel(ic, ic->ic_bsschan);
 		/* clear internal flags and any indication of a pick */
 		SCAN_PRIVATE(ss)->ss_iflags &= ~ISCAN_REP;
@@ -978,7 +978,7 @@ ieee80211_scan_dfs_action(struct ieee80211vap *vap,
 			  se->se_chan->ic_freq);
 		if (vap->iv_state == IEEE80211_S_RUN) {
 			ic->ic_chanchange_chan = se->se_chan->ic_ieee;
-			ic->ic_chanchange_tbtt = IEEE80211_RADAR_11HCOUNT;
+			ic->ic_chanchange_tbtt = IEEE80211_RADAR_CHANCHANGE_TBTT_COUNT;
 			ic->ic_flags |= IEEE80211_F_CHANSWITCH;
 		} else {
 			/* 
@@ -1015,7 +1015,7 @@ ieee80211_scan_dfs_action(struct ieee80211vap *vap,
 					  ic->ic_channels[chanStart].ic_ieee,
 					  ic->ic_channels[chanStart].ic_freq);
 				ic->ic_chanchange_chan = ic->ic_channels[chanStart].ic_ieee;
-				ic->ic_chanchange_tbtt = IEEE80211_RADAR_11HCOUNT;
+				ic->ic_chanchange_tbtt = IEEE80211_RADAR_CHANCHANGE_TBTT_COUNT;
 				ic->ic_flags |= IEEE80211_F_CHANSWITCH;
 			}
 		}
