@@ -131,7 +131,7 @@ static void skb_print_message(
 #else
 	printk("%s: %s%s:%d %s\n",
 #endif
-		DEV_NAME(skb->dev),
+		((skb != NULL) ? DEV_NAME(skb->dev) : "none"),
 		skb_count, 
 #ifdef IEEE80211_DEBUG_REFCNT
 		func1, line1,
@@ -268,7 +268,8 @@ static void print_skb_trackchange_message(
 
 static struct sk_buff *
 clean_clone_or_copy(struct sk_buff *skb) {
-	SKB_CB(skb)->tracked = 0;
+	if (skb != NULL)
+		SKB_CB(skb)->tracked = 0;
 	return skb;
 }
 
@@ -407,9 +408,9 @@ unref_skb(struct sk_buff *skb, int type,
 	}
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	if ((in_irq() || irqs_disabled()) 
-	     && (type == UNREF_USE_KFREE_SKB || type == UNREF_USE_DEV_KFREE_SKB)) 
-	{
+	if ((in_irq() || irqs_disabled()) && 
+			(type == UNREF_USE_KFREE_SKB || 
+			 type == UNREF_USE_DEV_KFREE_SKB)) {
 		skb_print_message(0 /* show_counter */, 
 			skb, func1, line1, func2, line2,
 			"ERROR: free an skb in interrupt context using a non-"
@@ -481,30 +482,26 @@ void ieee80211_dev_kfree_skb(struct sk_buff** pskb)
 {
 	struct sk_buff *skb;
 
-	/* Do not fail on null, we are going to use this in cleanup code */
+	/* Do not fail on null, as we are going to use this in cleanup code. */
 	if (!pskb || !(skb = *pskb))
 		return;
 
-	if (!skb_shared(skb)) {
-		/* Release the SKB references, for fragments of chain that are
-		 * unshared... starting at skb passed in. */
-		if (skb->prev == NULL) {
-			if (skb->next != NULL) {
-				skb->next->prev = NULL;
-			}
-			skb->next = NULL;
-		}
-		/* Release node reference, if any */
-		if (SKB_CB(skb)->ni != NULL) {
+	/* Release the SKB references, for fragments of chain that are
+	 * unshared... starting at skb passed in. */
+	if (skb->prev == NULL) {
+		if (skb->next != NULL)
+			skb->next->prev = NULL;
+		skb->next = NULL;
+	}
+
+	if (SKB_CB(skb)->ni != NULL) {
 #ifdef IEEE80211_DEBUG_REFCNT
-			ieee80211_unref_node_debug(&SKB_CB(skb)->ni, func, line);
+		ieee80211_unref_node_debug(&SKB_CB(skb)->ni, func, line);
 #else
-			ieee80211_unref_node(&SKB_CB(skb)->ni);
+		ieee80211_unref_node(&SKB_CB(skb)->ni);
 #endif
-		}
 	}
 
-	/* Decrement the ref count for the skb, possibly freeing the memory */
 #ifdef IEEE80211_DEBUG_REFCNT
 	unref_skb(skb, UNREF_USE_DEV_KFREE_SKB_ANY, 
 		  func, line, __func__, __LINE__);
