@@ -29,7 +29,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ieee80211_var.h 2915 2007-11-22 02:08:35Z mentor $
+ * $Id: ieee80211_var.h 3244 2008-01-25 00:29:30Z mtaylor $
  */
 
 #ifndef _NET80211_IEEE80211_VAR_H_
@@ -319,6 +319,7 @@ struct ieee80211com {
 	 */
 	int ic_nchans;				/* # entries in ic_channels */
 	struct ieee80211_channel ic_channels[IEEE80211_CHAN_MAX+1];
+	struct timeval ic_chan_non_occupy[IEEE80211_CHAN_MAX];
 	u_int8_t ic_chan_avail[IEEE80211_CHAN_BYTES];
 	u_int8_t ic_chan_active[IEEE80211_CHAN_BYTES];
 	struct ieee80211_channel *ic_curchan;	/* current channel */
@@ -440,6 +441,9 @@ struct ieee80211com {
 	unsigned int (*ic_write_register)(struct ieee80211com *, unsigned int, unsigned int);
 	unsigned int (*ic_read_register)(struct ieee80211com *, unsigned int, unsigned int*);
 #endif /* #ifdef ATH_REVERSE_ENGINEERING */
+
+	/* dump a lookup table for unobfuscating HAL function pointers */
+	unsigned int (*ic_dump_hal_map)(struct ieee80211com *);
 };
 
 #define MAX_PROC_IEEE80211_SIZE 16383
@@ -665,4 +669,16 @@ ieee80211_anyhdrspace(struct ieee80211com *ic, const void *data)
 		size = roundup(size, sizeof(u_int32_t));
 	return size;
 }
+
+/* Macros to print MAC address used in 802.11 headers */
+
+#define MAC_FMT "%02x:%02x:%02x:%02x:%02x:%02x"
+#define MAC_ADDR(addr) \
+  ((unsigned char *)(addr)) [0], \
+  ((unsigned char *)(addr)) [1], \
+  ((unsigned char *)(addr)) [2], \
+  ((unsigned char *)(addr)) [3], \
+  ((unsigned char *)(addr)) [4], \
+  ((unsigned char *)(addr)) [5]
+
 #endif /* _NET80211_IEEE80211_VAR_H_ */
